{*******************************************************}
{                                                       }
{       TBarScanThread component - Version 0.90         }
{                                                       }
{       This component is designed as a modular         }
{       encapsulation of the barcode scanner device     }
{       interface.                                      }
{                                                       }
{       This version still needs replete error          }
{       checking                                        }
{                                                       }
{       This version will NOT work under NT             }
{                                                       }
{                                                       }
{*******************************************************}
{                                                       }
{       04.22.97 - TF                                   }
{                                                       }
{*******************************************************}


unit BarScanT;

interface

uses Windows, Classes;

const
  MAX_BUF_LENGTH = 80;

type
  TShowDataProc=procedure(Data: string) of object;

  TBarScanThread=class(TThread)
  private
    FDCB: TDCB;
    FData: array[0..MAX_BUF_LENGTH] of Char;
    FDataStr: string;
    FDataLength: DWord;
    FStatus: Boolean;
    FConnected: Boolean;
    FComStat: TComStat;
    FShowDataProc: TShowDataProc;
    FHComm: THandle;
    procedure SetupConnection;
    procedure OpenConnection;
    procedure ReadData;
    procedure CloseConnection;
  protected
    procedure Execute; override;
  public
    constructor Create(ComNum: Integer; ShowDataProc: TShowDataProc);
    destructor Destroy; override;
    procedure ReleaseAndTerminate;
    property DataStr: string read FDataStr;
    property DataLength: Integer read FDataLength;
  end;

implementation

uses SysUtils;

procedure TBarScanThread.SetupConnection;
begin
  FDCB.DCBlength := SizeOf(FDCB);
  GetCommState(FHComm, FDCB);    // sets all the flow control flags automatically
  FDCB.BaudRate := 9600;
  FDCB.ByteSize := 8;
  FDCB.Parity := 0;
  FDCB.StopBits := 0;
  FStatus := SetCommState(FHComm, FDCB) ;
end;

constructor TBarScanThread.Create(ComNum: Integer; ShowDataProc: TShowDataProc);
begin
  FHComm := FileOpen(PChar('COM' + IntToStr(ComNum)), 0);
  FShowDataProc := ShowDataProc;    // proc to call on RXDATA
  inherited Create(False);          // creates thread non-suspended
end;

destructor TBarScanThread.Destroy;
begin
  ReleaseAndTerminate;             // detach from port or no harm done
  CloseConnection;                 // escape and purge or no harm done
  CloseHandle(FHComm);             // port handle is now invalid
end;

procedure TBarScanThread.ReadData;
var
  index: Integer;
  ErrorFlags: DWord;
begin
  ClearCommError(FHComm, ErrorFlags, @FComStat); // clue to size of data
  if FComStat.cbInQue < MAX_BUF_LENGTH then
    FDataLength := FComStat.cbInQue
  else
    FDataLength := MAX_BUF_LENGTH; // but don''t read more than max

  if (FDataLength > 0) then
    begin
      FStatus := ReadFile(FHComm, FData, FDataLength, FDataLength, nil) ;
      for index := 0 to FDataLength - 1 do
        FDataStr := FDataStr + FData[index]; // digest to string property
    end;
end;

procedure TBarScanThread.OpenConnection;
begin
  if FConnected then Exit;
  FStatus := SetCommMask(FHComm, EV_RXCHAR);  // mask for event
  FStatus := SetupComm(FHComm, 4096, 4096);   // buffer size
  FStatus := PurgeComm(FHComm, PURGE_TXABORT or PURGE_RXABORT or PURGE_TXCLEAR or PURGE_RXCLEAR) ;
  SetupConnection;                            // user settings
  FConnected := True;
end;

procedure TBarScanThread.CloseConnection;
begin
  FStatus := EscapeCommFunction(FHComm, CLRDTR) ;
  FStatus := PurgeComm(FHComm, PURGE_TXABORT or PURGE_RXABORT or PURGE_TXCLEAR or PURGE_RXCLEAR);
  FConnected := False;
end;

procedure TBarScanThread.Execute;
var
  EvtMask: DWord;
begin
  OpenConnection;                                       // assume unopened
  repeat
    EvtMask := 0 ;                                      // compares with EV_RXCHAR
    FStatus := WaitCommEvent(FHComm, EvtMask, nil);     // blocks until data
    if (EvtMask and EV_RXCHAR) = EV_RXCHAR then
      begin
        repeat
          ReadData;                                       // clears buffer in n-byte segments
        until FDataLength <= 0;                           // scanner has completed barcode
      if Pos(#13, FDataStr) > 0 then
        begin
          Delete(FDataStr, Length(FDataStr), 1);          // digest EOL
          if Assigned(FShowDataProc) then
            FShowDataProc(FDataStr);                      // put data wherever user wants (notice: no Synchronize)
          FDataStr := '';                                 // clear digested buffer
        end;
    end;
  until Terminated;
  CloseConnection;
end;

procedure TBarScanThread.ReleaseAndTerminate;            // call this instead of Terminate or the
begin                                                 // thread will never release the com port
  Terminate;
  SetCommMask(FHComm, 0);
end;

end.



